@@ -6,23 +6,31 @@
 import copy
 import grp
 import inspect
-import optparse
+try:
+    import argparse
+except ImportError: # python 2.6
+    from . import argparse_compat as argparse
 import os
 import pwd
+import sys
 import textwrap
 import types
 
 from gunicorn import __version__
 from gunicorn.errors import ConfigError
+from gunicorn import six
 from gunicorn import util
 
 KNOWN_SETTINGS = []
+PLATFORM = sys.platform
+
 
 def wrap_method(func):
     def _wrapped(instance, *args, **kwargs):
         return func(*args, **kwargs)
     return _wrapped
 
+
 def make_settings(ignore=None):
     settings = {}
     ignore = ignore or ()
@@ -33,11 +41,13 @@ def make_settings(ignore=None):
         settings[setting.name] = setting.copy()
     return settings
 
+
 class Config(object):
 
-    def __init__(self, usage=None):
+    def __init__(self, usage=None, prog=None):
         self.settings = make_settings()
         self.usage = usage
+        self.prog = prog or sys.argv[0]
 
     def __getattr__(self, name):
         if name not in self.settings:
@@ -57,16 +67,22 @@ class Config(object):
     def parser(self):
         kwargs = {
             "usage": self.usage,
-            "version": __version__
+            "version": "%(prog)s (version " +  __version__ + ")\n",
+            "prog": self.prog
         }
-        parser = optparse.OptionParser(**kwargs)
+        parser = argparse.ArgumentParser(**kwargs)
+        parser.add_argument("args", nargs="*", help=argparse.SUPPRESS)
 
-        keys = self.settings.keys()
+        keys = list(self.settings)
         def sorter(k):
             return (self.settings[k].section, self.settings[k].order)
-        keys.sort(key=sorter)
+
+        keys = sorted(self.settings, key=self.settings.__getitem__)
         for k in keys:
             self.settings[k].add_option(parser)
+
+
+
         return parser
 
     @property
@@ -83,8 +99,8 @@ class Config(object):
 
     @property
     def address(self):
-        bind = self.settings['bind'].get()
-        return util.parse_address(util.to_bytestring(bind))
+        s = self.settings['bind'].get()
+        return [util.parse_address(six.bytes_to_str(bind)) for bind in s]
 
     @property
     def uid(self):
@@ -112,6 +128,21 @@ class Config(object):
             logger_class.install()
         return logger_class
 
+    @property
+    def is_ssl(self):
+        return self.certfile or self.keyfile
+
+    @property
+    def ssl_options(self):
+        opts = {}
+        if self.certfile:
+            opts['certfile'] = self.certfile
+
+        if self.keyfile:
+            opts['keyfile'] = self.keyfile
+
+        return opts
+
 
 class SettingMeta(type):
     def __new__(cls, name, bases, attrs):
@@ -133,9 +164,8 @@ class SettingMeta(type):
         setattr(cls, "desc", desc)
         setattr(cls, "short", desc.splitlines()[0])
 
-class Setting(object):
-    __metaclass__ = SettingMeta
 
+class Setting(object):
     name = None
     value = None
     section = None
@@ -147,6 +177,10 @@ class Setting(object):
     default = None
     short = None
     desc = None
+    nargs = None
+    const = None
+
+
 
     def __init__(self):
         if self.default is not None:
@@ -156,17 +190,31 @@ class Setting(object):
         if not self.cli:
             return
         args = tuple(self.cli)
+
+        help_txt = "%s [%s]" % (self.short, self.default)
+        help_txt = help_txt.replace("%", "%%")
+
         kwargs = {
             "dest": self.name,
-            "metavar": self.meta or None,
             "action": self.action or "store",
-            "type": self.type or "string",
+            "type": self.type or str,
             "default": None,
-            "help": "%s [%s]" % (self.short, self.default)
+            "help": help_txt
         }
+
+        if self.meta is not None:
+            kwargs['metavar'] = self.meta
+
         if kwargs["action"] != "store":
             kwargs.pop("type")
-        parser.add_option(*args, **kwargs)
+
+        if self.nargs is not None:
+            kwargs["nargs"] = self.nargs
+
+        if self.const is not None:
+            kwargs["const"] = self.const
+
+        parser.add_argument(*args, **kwargs)
 
     def copy(self):
         return copy.copy(self)
@@ -175,13 +223,21 @@ class Setting(object):
         return self.value
 
     def set(self, val):
-        assert callable(self.validator), "Invalid validator: %s" % self.name
+        assert six.callable(self.validator), "Invalid validator: %s" % self.name
         self.value = self.validator(val)
 
+    def __lt__(self, other):
+        return (self.section == other.section and
+                self.order < other.order)
+    __cmp__ = __lt__
+
+Setting = SettingMeta('Setting', (Setting,), {})
+
+
 def validate_bool(val):
-    if isinstance(val, types.BooleanType):
+    if isinstance(val, bool):
         return val
-    if not isinstance(val, basestring):
+    if not isinstance(val, six.string_types):
         raise TypeError("Invalid type for casting: %s" % val)
     if val.lower().strip() == "true":
         return True
@@ -190,13 +246,15 @@ def validate_bool(val):
     else:
         raise ValueError("Invalid boolean: %s" % val)
 
+
 def validate_dict(val):
     if not isinstance(val, dict):
         raise TypeError("Value is not a dictionary: %s " % val)
     return val
 
+
 def validate_pos_int(val):
-    if not isinstance(val, (types.IntType, types.LongType)):
+    if not isinstance(val, six.integer_types):
         val = int(val, 0)
     else:
         # Booleans are ints!
@@ -205,13 +263,35 @@ def validate_pos_int(val):
         raise ValueError("Value must be positive: %s" % val)
     return val
 
+
 def validate_string(val):
     if val is None:
         return None
-    if not isinstance(val, basestring):
+    if not isinstance(val, six.string_types):
         raise TypeError("Not a string: %s" % val)
     return val.strip()
 
+
+def validate_list_string(val):
+    if not val:
+        return []
+
+    # legacy syntax
+    if isinstance(val, six.string_types):
+        val = [val]
+
+    return [validate_string(v) for v in val]
+
+
+def validate_string_to_list(val):
+    val = validate_string(val)
+
+    if not val:
+        return []
+
+    return [v.strip() for v in val.split(",") if v]
+
+
 def validate_class(val):
     if inspect.isfunction(val) or inspect.ismethod(val):
         val = val()
@@ -219,11 +299,26 @@ def validate_class(val):
         return val
     return validate_string(val)
 
+
 def validate_callable(arity):
     def _validate_callable(val):
-        if not callable(val):
-            raise TypeError("Value is not callable: %s" % val)
-        if arity != len(inspect.getargspec(val)[0]):
+        if isinstance(val, six.string_types):
+            try:
+                mod_name, obj_name = val.rsplit(".", 1)
+            except ValueError:
+                raise TypeError("Value '%s' is not import string. "
+                                "Format: module[.submodules...].object" % val)
+            try:
+                mod = __import__(mod_name, fromlist=[obj_name])
+                val = getattr(mod, obj_name)
+            except ImportError as e:
+                raise TypeError(str(e))
+            except AttributeError:
+                raise TypeError("Can not load '%s' from '%s'"
+                    "" % (obj_name, mod_name))
+        if not six.callable(val):
+            raise TypeError("Value is not six.callable: %s" % val)
+        if arity != -1 and arity != len(inspect.getargspec(val)[0]):
             raise TypeError("Value must have an arity of: %s" % arity)
         return val
     return _validate_callable
@@ -242,6 +337,7 @@ def validate_user(val):
         except KeyError:
             raise ConfigError("No such user: '%s'" % val)
 
+
 def validate_group(val):
     if val is None:
         return os.getegid()
@@ -256,25 +352,19 @@ def validate_group(val):
         except KeyError:
             raise ConfigError("No such group: '%s'" % val)
 
+
 def validate_post_request(val):
-
-    # decorator
-    def wrap_post_request(fun):
-        def _wrapped(instance, req, environ):
-            return fun(instance, req)
-        return _wrapped
-
-    if not callable(val):
-        raise TypeError("Value isn't a callable: %s" % val)
+    val = validate_callable(-1)(val)
 
     largs = len(inspect.getargspec(val)[0])
-    if largs == 3:
+    if largs == 4:
         return val
+    elif largs == 3:
+        return lambda worker, req, env, _r: val(worker, req, env)
     elif largs == 2:
-        return wrap_post_request(val)
+        return lambda worker, req, _e, _r: val(worker, req)
     else:
-        raise TypeError("Value must have an arity of: 3")
-
+        raise TypeError("Value must have an arity of: 4")
 
 
 class ConfigFile(Setting):
@@ -291,27 +381,42 @@ class ConfigFile(Setting):
         application specific configuration.
         """
 
+
 class Bind(Setting):
     name = "bind"
+    action = "append"
     section = "Server Socket"
     cli = ["-b", "--bind"]
     meta = "ADDRESS"
-    validator = validate_string
-    default = "127.0.0.1:8000"
+    validator = validate_list_string
+
+    if 'PORT' in os.environ:
+        default = ['0.0.0.0:{0}'.format(os.environ.get('PORT'))]
+    else:
+        default = ['127.0.0.1:8000']
+
     desc = """\
         The socket to bind.
 
         A string of the form: 'HOST', 'HOST:PORT', 'unix:PATH'. An IP is a valid
         HOST.
+
+        Multiple addresses can be bound. ex.::
+
+            $ gunicorn -b 127.0.0.1:8000 -b [::1]:8000 test:app
+
+        will bind the `test:app` application on localhost both on ipv6
+        and ipv4 interfaces.
         """
 
+
 class Backlog(Setting):
     name = "backlog"
     section = "Server Socket"
     cli = ["--backlog"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 2048
     desc = """\
         The maximum number of pending connections.
@@ -324,13 +429,14 @@ class Backlog(Setting):
         Must be a positive integer. Generally set in the 64-2048 range.
         """
 
+
 class Workers(Setting):
     name = "workers"
     section = "Worker Processes"
     cli = ["-w", "--workers"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 1
     desc = """\
         The number of worker process for handling requests.
@@ -340,6 +446,7 @@ class Workers(Setting):
         application's work load.
         """
 
+
 class WorkerClass(Setting):
     name = "worker_class"
     section = "Worker Processes"
@@ -368,13 +475,14 @@ class WorkerClass(Setting):
         can also load the gevent class with ``egg:gunicorn#gevent``
         """
 
+
 class WorkerConnections(Setting):
     name = "worker_connections"
     section = "Worker Processes"
     cli = ["--worker-connections"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 1000
     desc = """\
         The maximum number of simultaneous clients.
@@ -382,13 +490,14 @@ class WorkerConnections(Setting):
         This setting only affects the Eventlet and Gevent worker types.
         """
 
+
 class MaxRequests(Setting):
     name = "max_requests"
     section = "Worker Processes"
     cli = ["--max-requests"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 0
     desc = """\
         The maximum number of requests a worker will process before restarting.
@@ -401,13 +510,14 @@ class MaxRequests(Setting):
         restarts are disabled.
         """
 
+
 class Timeout(Setting):
     name = "timeout"
     section = "Worker Processes"
     cli = ["-t", "--timeout"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 30
     desc = """\
         Workers silent for more than this many seconds are killed and restarted.
@@ -418,13 +528,14 @@ class Timeout(Setting):
         is not tied to the length of time required to handle a single request.
         """
 
+
 class GracefulTimeout(Setting):
     name = "graceful_timeout"
     section = "Worker Processes"
     cli = ["--graceful-timeout"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 30
     desc = """\
         Timeout for graceful workers restart.
@@ -434,13 +545,14 @@ class GracefulTimeout(Setting):
         be force killed.
         """
 
+
 class Keepalive(Setting):
     name = "keepalive"
     section = "Worker Processes"
     cli = ["--keep-alive"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 2
     desc = """\
         The number of seconds to wait for requests on a Keep-Alive connection.
@@ -448,13 +560,14 @@ class Keepalive(Setting):
         Generally set in the 1-5 seconds range.
         """
 
+
 class LimitRequestLine(Setting):
     name = "limit_request_line"
     section = "Security"
     cli = ["--limit-request-line"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 4094
     desc = """\
         The maximum size of HTTP request line in bytes.
@@ -471,15 +584,16 @@ class LimitRequestLine(Setting):
         This parameter can be used to prevent any DDOS attack.
         """
 
+
 class LimitRequestFields(Setting):
     name = "limit_request_fields"
     section = "Security"
     cli = ["--limit-request-fields"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 100
-    desc= """\
+    desc = """\
         Limit the number of HTTP headers fields in a request.
 
         This parameter is used to limit the number of headers in a request to
@@ -488,21 +602,23 @@ class LimitRequestFields(Setting):
         32768.
         """
 
+
 class LimitRequestFieldSize(Setting):
     name = "limit_request_field_size"
     section = "Security"
     cli = ["--limit-request-field_size"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 8190
-    desc= """\
+    desc = """\
         Limit the allowed size of an HTTP request header field.
 
         Value is a number from 0 (unlimited) to 8190. to set the limit
         on the allowed size of an HTTP request header field.
         """
 
+
 class Debug(Setting):
     name = "debug"
     section = "Debugging"
@@ -517,6 +633,7 @@ class Debug(Setting):
         handling that's sent to clients.
         """
 
+
 class Spew(Setting):
     name = "spew"
     section = "Debugging"
@@ -530,10 +647,11 @@ class Spew(Setting):
         This is the nuclear option.
         """
 
+
 class ConfigCheck(Setting):
     name = "check_config"
     section = "Debugging"
-    cli = ["--check-config",]
+    cli = ["--check-config", ]
     validator = validate_bool
     action = "store_true"
     default = False
@@ -541,6 +659,7 @@ class ConfigCheck(Setting):
         Check the configuration..
         """
 
+
 class PreloadApp(Setting):
     name = "preload_app"
     section = "Server Mechanics"
@@ -557,6 +676,7 @@ class PreloadApp(Setting):
         restarting workers.
         """
 
+
 class Daemon(Setting):
     name = "daemon"
     section = "Server Mechanics"
@@ -571,6 +691,7 @@ class Daemon(Setting):
         background.
         """
 
+
 class Pidfile(Setting):
     name = "pidfile"
     section = "Server Mechanics"
@@ -584,6 +705,7 @@ class Pidfile(Setting):
         If not set, no PID file will be written.
         """
 
+
 class User(Setting):
     name = "user"
     section = "Server Mechanics"
@@ -599,6 +721,7 @@ class User(Setting):
         the worker process user.
         """
 
+
 class Group(Setting):
     name = "group"
     section = "Server Mechanics"
@@ -614,13 +737,14 @@ class Group(Setting):
         the worker processes group.
         """
 
+
 class Umask(Setting):
     name = "umask"
     section = "Server Mechanics"
     cli = ["-m", "--umask"]
     meta = "INT"
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 0
     desc = """\
         A bit mask for the file mode on files written by Gunicorn.
@@ -632,6 +756,7 @@ class Umask(Setting):
         "0xFF", "0022" are valid for decimal, hex, and octal representations)
         """
 
+
 class TmpUploadDir(Setting):
     name = "tmp_upload_dir"
     section = "Server Mechanics"
@@ -648,12 +773,14 @@ class TmpUploadDir(Setting):
         temporary directory.
         """
 
+
 class SecureSchemeHeader(Setting):
     name = "secure_scheme_headers"
     section = "Server Mechanics"
     validator = validate_dict
     default = {
         "X-FORWARDED-PROTOCOL": "ssl",
+        "X-FORWARDED-PROTO": "https",
         "X-FORWARDED-SSL": "on"
     }
     desc = """\
@@ -671,6 +798,8 @@ class SecureSchemeHeader(Setting):
         It is important that your front-end proxy configuration ensures that
         the headers defined here can not be passed directly from the client.
         """
+
+
 class XForwardedFor(Setting):
     name = "x_forwarded_for_header"
     section = "Server Mechanics"
@@ -682,6 +811,23 @@ class XForwardedFor(Setting):
         address of the client connection to gunicorn via a proxy.
         """
 
+
+class ForwardedAllowIPS(Setting):
+    name = "forwarded_allow_ips"
+    section = "Server Mechanics"
+    meta = "STRING"
+    validator = validate_string_to_list
+    default = "127.0.0.1"
+    desc = """\
+        Front-end's IPs from which allowed to handle X-Forwarded-* headers.
+        (comma separate).
+
+        Set to "*" to disable checking of Front-end IPs (useful for setups
+        where you don't know in advance the IP address of Front-end, but
+        you still trust the environment)
+        """
+
+
 class AccessLog(Setting):
     name = "accesslog"
     section = "Logging"
@@ -695,6 +841,7 @@ class AccessLog(Setting):
         "-" means log to stderr.
         """
 
+
 class AccessLogFormat(Setting):
     name = "access_log_format"
     section = "Logging"
@@ -726,6 +873,7 @@ class AccessLogFormat(Setting):
         {Header}o: response header
         """
 
+
 class ErrorLog(Setting):
     name = "errorlog"
     section = "Logging"
@@ -739,6 +887,7 @@ class ErrorLog(Setting):
         "-" means log to stderr.
         """
 
+
 class Loglevel(Setting):
     name = "loglevel"
     section = "Logging"
@@ -758,6 +907,7 @@ class Loglevel(Setting):
         * critical
         """
 
+
 class LoggerClass(Setting):
     name = "logger_class"
     section = "Logging"
@@ -791,6 +941,7 @@ Gunicorn uses the standard Python logging module's Configuration
 file format.
 """
 
+
 class Procname(Setting):
     name = "proc_name"
     section = "Process Naming"
@@ -809,6 +960,7 @@ class Procname(Setting):
         It defaults to 'gunicorn'.
         """
 
+
 class DefaultProcName(Setting):
     name = "default_proc_name"
     section = "Process Naming"
@@ -833,9 +985,10 @@ class DjangoSettings(Setting):
         DJANGO_SETTINGS_MODULE environment variable will be used.
         """
 
-class DjangoPythonPath(Setting):
+
+class PythonPath(Setting):
     name = "pythonpath"
-    section = "Django"
+    section = "Server Mechanics"
     cli = ["--pythonpath"]
     meta = "STRING"
     validator = validate_string
@@ -847,11 +1000,13 @@ class DjangoPythonPath(Setting):
         '/home/djangoprojects/myproject'.
         """
 
+
 class OnStarting(Setting):
     name = "on_starting"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = "callable"
+    type = six.callable
+
     def on_starting(server):
         pass
     default = staticmethod(on_starting)
@@ -861,11 +1016,13 @@ class OnStarting(Setting):
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
+
 class OnReload(Setting):
     name = "on_reload"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = "callable"
+    type = six.callable
+
     def on_reload(server):
         pass
     default = staticmethod(on_reload)
@@ -875,11 +1032,13 @@ class OnReload(Setting):
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
+
 class WhenReady(Setting):
     name = "when_ready"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = "callable"
+    type = six.callable
+
     def when_ready(server):
         pass
     default = staticmethod(when_ready)
@@ -889,11 +1048,13 @@ class WhenReady(Setting):
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
+
 class Prefork(Setting):
     name = "pre_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = "callable"
+    type = six.callable
+
     def pre_fork(server, worker):
         pass
     default = staticmethod(pre_fork)
@@ -904,11 +1065,13 @@ class Prefork(Setting):
         new Worker.
         """
 
+
 class Postfork(Setting):
     name = "post_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = "callable"
+    type = six.callable
+
     def post_fork(server, worker):
         pass
     default = staticmethod(post_fork)
@@ -919,11 +1082,13 @@ class Postfork(Setting):
         new Worker.
         """
 
+
 class PreExec(Setting):
     name = "pre_exec"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = "callable"
+    type = six.callable
+
     def pre_exec(server):
         pass
     default = staticmethod(pre_exec)
@@ -933,11 +1098,13 @@ class PreExec(Setting):
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
+
 class PreRequest(Setting):
     name = "pre_request"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = "callable"
+    type = six.callable
+
     def pre_request(worker, req):
         worker.log.debug("%s %s" % (req.method, req.path))
     default = staticmethod(pre_request)
@@ -948,12 +1115,14 @@ class PreRequest(Setting):
         the Request.
         """
 
+
 class PostRequest(Setting):
     name = "post_request"
     section = "Server Hooks"
     validator = validate_post_request
-    type = "callable"
-    def post_request(worker, req, environ):
+    type = six.callable
+
+    def post_request(worker, req, environ, resp):
         pass
     default = staticmethod(post_request)
     desc = """\
@@ -963,11 +1132,13 @@ class PostRequest(Setting):
         the Request.
         """
 
+
 class WorkerExit(Setting):
     name = "worker_exit"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = "callable"
+    type = six.callable
+
     def worker_exit(server, worker):
         pass
     default = staticmethod(worker_exit)
@@ -977,3 +1148,132 @@ class WorkerExit(Setting):
         The callable needs to accept two instance variables for the Arbiter and
         the just-exited Worker.
         """
+
+
+class NumWorkersChanged(Setting):
+    name = "nworkers_changed"
+    section = "Server Hooks"
+    validator = validate_callable(3)
+    type = six.callable
+
+    def nworkers_changed(server, new_value, old_value):
+        pass
+    default = staticmethod(nworkers_changed)
+    desc = """\
+        Called just after num_workers has been changed.
+
+        The callable needs to accept an instance variable of the Arbiter and
+        two integers of number of workers after and before change.
+
+        If the number of workers is set for the first time, old_value would be
+        None.
+        """
+
+
+class ProxyProtocol(Setting):
+    name = "proxy_protocol"
+    section = "Server Mechanics"
+    cli = ["--proxy-protocol"]
+    validator = validate_bool
+    default = False
+    action = "store_true"
+    desc = """\
+        Enable detect PROXY protocol (PROXY mode).
+
+        Allow using Http and Proxy together. It's may be useful for work with
+        stunnel as https frondend and gunicorn as http server.
+
+        PROXY protocol: http://haproxy.1wt.eu/download/1.5/doc/proxy-protocol.txt
+
+        Example for stunnel config::
+
+        [https]
+        protocol = proxy
+        accept  = 443
+        connect = 80
+        cert = /etc/ssl/certs/stunnel.pem
+        key = /etc/ssl/certs/stunnel.key
+        """
+
+
+class ProxyAllowFrom(Setting):
+    name = "proxy_allow_ips"
+    section = "Server Mechanics"
+    cli = ["--proxy-allow-from"]
+    validator = validate_string_to_list
+    default = "127.0.0.1"
+    desc = """\
+        Front-end's IPs from which allowed accept proxy requests (comma separate).
+        """
+
+
+class KeyFile(Setting):
+    name = "keyfile"
+    section = "Ssl"
+    cli = ["--keyfile"]
+    meta = "FILE"
+    validator = validate_string
+    default = None
+    desc = """\
+    SSL key file
+    """
+
+
+class CertFile(Setting):
+    name = "certfile"
+    section = "Ssl"
+    cli = ["--certfile"]
+    meta = "FILE"
+    validator = validate_string
+    default = None
+    desc = """\
+    SSL certificate file
+    """
+
+
+class SyslogTo(Setting):
+    name = "syslog_addr"
+    section = "Logging"
+    cli = ["--log-syslog-to"]
+    meta = "SYSLOG_ADDR"
+    validator = validate_string
+
+    if PLATFORM == "darwin":
+        default = "unix:///var/run/syslog"
+    elif PLATFORM in ('freebsd', 'dragonfly', ):
+        default = "unix:///var/run/log"
+    elif PLATFORM == "openbsd":
+        default = "unix:///dev/log"
+    else:
+        default = "udp://localhost:514"
+
+    desc = """\
+    Address to send syslog messages
+    """
+
+
+class Syslog(Setting):
+    name = "syslog"
+    section = "Logging"
+    cli = ["--log-syslog"]
+    validator = validate_bool
+    action = 'store_true'
+    default = False
+    desc = """\
+    Log to syslog.
+    """
+
+
+class SyslogPrefix(Setting):
+    name = "syslog_prefix"
+    section = "Logging"
+    cli = ["--log-syslog-prefix"]
+    meta = "SYSLOG_PREFIX"
+    validator = validate_string
+    default = None
+    desc = """\
+    makes gunicorn use the parameter as program-name in the syslog entries.
+
+    All entries will be prefixed by gunicorn.<prefix>. By default the program
+    name is the name of the process.
+    """
